import unittest
import requests
from unittest.mock import patch, Mock
from passinfo_sdk.client import PassInfoSDKClient
from passinfo_sdk.exceptions import PassInfoAPIError

class TestPassInfoSDKClient(unittest.TestCase):
    def setUp(self):
        self.api_key = "417a5816-c0ff-4954-879e-dfe969ec00fd"
        self.client_id = 13
        self.client = PassInfoSDKClient(
            api_key=self.api_key,
            client_id=self.client_id
        )

    @patch('passinfo_sdk.client.requests.request')
    def test_send_message_success(self, mock_request):
        # Arrange
        mock_response = Mock()
        mock_response.json.return_value = {"status": "success"}
        mock_response.status_code = 200
        mock_request.return_value = mock_response

        # Act
        response = self.client.send_message(
            message="Hello World",
            contact="622915547",
            sender_name="NEKOMAT"
        )

        # Assert
        mock_request.assert_called_once_with(
            method='POST',
            url='http://api.passinfo.net/api/dev/message/single_message',
            headers={
                'X-API-Key': self.api_key,
                'X-Client-ID': self.client_id,
                'Content-Type': 'application/json'
            },
            params=None,
            json={
                'message': 'Hello World',
                'contact': '622915547',
                'senderName': 'NEKOMAT'
            }
        )
        self.assertEqual(response, {"status": "success"})

    def test_send_message_validation(self):
        # Test missing message
        with self.assertRaises(PassInfoAPIError) as context:
            self.client.send_message(message=None, contact="622915547", sender_name="NEKOMAT")
        self.assertEqual(str(context.exception.message), "Message is required.")

        # Test missing contact
        with self.assertRaises(PassInfoAPIError) as context:
            self.client.send_message(message="Hello", contact=None, sender_name="NEKOMAT")
        self.assertEqual(str(context.exception.message), "Contact is required.")

        # Test missing sender name
        with self.assertRaises(PassInfoAPIError) as context:
            self.client.send_message(message="Hello", contact="622915547", sender_name=None)
        self.assertEqual(str(context.exception.message), "Sender name is required.")

    @patch('passinfo_sdk.client.requests.request')
    def test_send_message_api_error(self, mock_request):
        # Arrange
        mock_request.side_effect = requests.exceptions.RequestException("API Error")

        # Act & Assert
        with self.assertRaises(PassInfoAPIError) as context:
            self.client.send_message(
                message="Hello World",
                contact="622915547",
                sender_name="NEKOMAT"
            )
        self.assertIn("API request failed", str(context.exception.message))

    @patch('passinfo_sdk.client.requests.request')
    def test_send_message_bulk_success(self, mock_request):
        # Arrange
        mock_response = Mock()
        mock_response.json.return_value = {"status": "success"}
        mock_response.status_code = 200
        mock_request.return_value = mock_response
        contacts = ["622915547", "622915548"]

        # Act
        response = self.client.send_message_bulk(
            message="Hello World",
            sender_name="NEKOMAT",
            contacts=contacts
        )

        # Assert
        mock_request.assert_called_once()
        self.assertEqual(response, {"status": "success"})

    @patch('passinfo_sdk.client.requests.request')
    def test_send_message_group_success(self, mock_request):
        # Arrange
        mock_response = Mock()
        mock_response.json.return_value = {"status": "success"}
        mock_response.status_code = 200
        mock_request.return_value = mock_response
        group_id = "group123"

        # Act
        response = self.client.send_message_group(
            message="Hello World",
            sender_name="NEKOMAT",
            group_id=group_id
        )

        # Assert
        mock_request.assert_called_once()
        self.assertEqual(response, {"status": "success"})

if __name__ == '__main__':
    unittest.main()